import memoize from '@essentials/memoize-one';
import cloneDeep from 'clone-deep';
import cssToRN from 'css-to-react-native';
import * as React from 'react';
import * as RN from 'react-native';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var __reactCreateElement__ = React.createElement;
var emptyObj = {};

function tokensAreEqual(_ref, _ref2) {
  var [t] = _ref;
  var [nt] = _ref2;
  return t === nt;
}

function _ref9(current) {
  return ++current;
}

function _ref10(current) {
  return ++current;
}

function createStyles(options) {
  var _options$themes, _options$tokens;

  if (options === void 0) {
    options = emptyObj;
  }

  var themes = (_options$themes = options.themes) !== null && _options$themes !== void 0 ? _options$themes : emptyObj;
  var currentTheme = "default";
  var tokens = {
    default: (_options$tokens = options.tokens) !== null && _options$tokens !== void 0 ? _options$tokens : emptyObj
  };

  if (options.themes) {
    var themeNames = Object.keys(options.themes);
    currentTheme = themeNames[0];

    for (var themeName of themeNames) {
      tokens[themeName] = mergeTokens(cloneDeep(tokens.default), themes[themeName]);
    }

    var colorScheme = RN.Appearance.getColorScheme();

    if (themeNames.length === 2 && "light" in tokens && "dark" in tokens && colorScheme && colorScheme in themes) {
      currentTheme = colorScheme;
    }
  }

  var isAutoThemeable = Object.values(tokens).length === 3 && "light" in tokens && "dark" in tokens;

  function cls(literals) {
    literals = typeof literals === "function" ? memoize(literals) : literals;

    for (var _len = arguments.length, placeholders = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      placeholders[_key - 1] = arguments[_key];
    }

    var styles = compileStyles(compileLiterals(literals, ...placeholders), tokens[currentTheme]);

    return styles;
  }

  var styles = {
    variants(styleMap) {
      var memoMap = {};

      for (var key in styleMap) {
        // @ts-expect-error: key exists
        memoMap[key] = typeof styleMap[key] === "function" ? // @ts-expect-error: is assignable
        memoize(styleMap[key], tokensAreEqual) : styleMap[key];
      } // style('text', {})


      function style() {
        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        var numArgs = args.length;
        var sheet = memoMap.default ? compileStyles(memoMap.default, tokens[currentTheme]) : {};

        if (numArgs === 1 && typeof args[0] === "string") {
          sheet = _extends({}, sheet, compileStyles(memoMap[args[0]], tokens[currentTheme]));
        } else if (numArgs > 0) {
          var i = 0;
          var arg;
          sheet = _extends({}, sheet);

          for (; i < numArgs; i++) {
            arg = args[i];

            if (typeof arg === "string") {
              Object.assign(sheet, compileStyles(memoMap[arg], tokens[currentTheme]));
            } else if (typeof arg === "object") {
              for (var _key3 in arg) {
                if (arg[_key3]) Object.assign(sheet, compileStyles(memoMap[_key3], tokens[currentTheme]));
              }
            }
          }
        }

        return sheet;
      }

      style.styles = memoMap;
      return style;
    },

    one(literals) {
      literals = typeof literals === "function" ? memoize(literals, tokensAreEqual) : literals;

      for (var _len3 = arguments.length, placeholders = new Array(_len3 > 1 ? _len3 - 1 : 0), _key4 = 1; _key4 < _len3; _key4++) {
        placeholders[_key4 - 1] = arguments[_key4];
      }

      var compiledLiterals = compileLiterals(literals, ...placeholders);
      return function oneStyle(createStyle) {
        if (!createStyle && createStyle !== void 0) return emptyObj;
        var styles = compileStyles(compiledLiterals, tokens[currentTheme]);

        return styles;
      };
    },

    cls,

    lazy(lazyFn) {
      return function (value) {
        if (value === undefined) return emptyObj;
        var styles = compileStyles(lazyFn(value), tokens[currentTheme]);

        return styles;
      };
    },

    join() {
      return cls("".concat(...arguments));
    },

    tokens,
    themes
  };
  var DashContext = /*#__PURE__*/React.createContext({
    styles,
    tokens: tokens[currentTheme],
    theme: currentTheme,

    setTheme(theme) {
      /* istanbul ignore next */
      console.error("DashContext was consumed outside of a Provider");
    },

    insertTokens(nextTokens) {
      /* istanbul ignore next */
      console.error("DashContext was consumed outside of a Provider");
    },

    insertThemes(nextThemes) {
      /* istanbul ignore next */
      console.error("DashContext was consumed outside of a Provider");
    }

  });
  /**
   * A hook for consuming dash context from the provider
   */

  function useDash() {
    return React.useContext(DashContext);
  }

  function styled(Component, baseStyles, areEqual) {
    var styles = typeof baseStyles === "function" && areEqual ? memoize(baseStyles, Array.isArray(areEqual) ? areEqual.length === 1 ? (_ref3, _ref4) => {
      var [t, p] = _ref3;
      var [nt, np] = _ref4;
      return t === nt && p[areEqual[0]] === np[areEqual[1]];
    } : (_ref5, _ref6) => {
      var [t, p] = _ref5;
      var [nt, np] = _ref6;
      return t === nt && areEqual.every(key => p[key] === np[key]);
    } : areEqual) : baseStyles;

    function compileRecursiveStyles(style, tokens) {
      function _ref8(s) {
        return compileRecursiveStyles(s, tokens);
      }

      if (typeof style === "function" || typeof style === "string") {
        return compileStyles(style, tokens);
      } else if (Array.isArray(style)) {
        return Object.assign({}, ...style.map(_ref8));
      }

      return style || {};
    }

    var RefForwardingComponent = /*#__PURE__*/React.forwardRef((props, ref) => {
      var dash = useDash();
      var baseStyle = !styles || typeof styles === "object" ? styles : compileStyles(typeof styles === "function" ? tokens => styles(tokens, props) : styles, dash.tokens);
      var outerStyle = typeof props.style === "function" || typeof props.style === "string" ? compileStyles(props.style, dash.tokens) : Array.isArray(props.style) ? compileRecursiveStyles(props.style, dash.tokens) : props.style;
      return /*#__PURE__*/__reactCreateElement__(Component, _extends({}, props, {
        style: outerStyle && baseStyle ? _extends({}, baseStyle, outerStyle) : outerStyle ? outerStyle : baseStyle,
        ref: ref
      }));
    });
    return RefForwardingComponent;
  }

  return {
    styles,
    styled,
    useDash,

    DashProvider(_ref7) {
      var {
        theme: controlledTheme,
        defaultTheme,
        onThemeChange,
        disableAutoThemeChange,
        children
      } = _ref7;
      var colorScheme = RN.useColorScheme();
      var didMount = React.useRef(false);
      var [userTheme, setTheme] = React.useState(() => {
        if (controlledTheme) {
          currentTheme = controlledTheme;
          return controlledTheme;
        }

        if (defaultTheme) {
          currentTheme = defaultTheme;
          return defaultTheme;
        }

        if (colorScheme && colorScheme in tokens && isAutoThemeable) {
          currentTheme = colorScheme;
          return colorScheme;
        }

        return currentTheme;
      });
      var [tokenPointer, setTokenPointer] = React.useState(0);
      var theme = controlledTheme !== null && controlledTheme !== void 0 ? controlledTheme : userTheme;
      currentTheme = theme;
      var storedOnChange = React.useRef(onThemeChange);
      React.useLayoutEffect(() => {
        storedOnChange.current = onThemeChange;
      });
      React.useLayoutEffect(() => {
        if (controlledTheme && controlledTheme !== userTheme) {
          setTheme(controlledTheme);
        }
      }, [controlledTheme, userTheme]);
      React.useLayoutEffect(() => {
        if (didMount.current && !controlledTheme && colorScheme && colorScheme !== theme && isAutoThemeable && !disableAutoThemeChange) {
          var _storedOnChange$curre;

          currentTheme = colorScheme;
          setTheme(currentTheme);
          (_storedOnChange$curre = storedOnChange.current) === null || _storedOnChange$curre === void 0 ? void 0 : _storedOnChange$curre.call(storedOnChange, currentTheme);
        }

        didMount.current = true;
      }, [colorScheme, controlledTheme]);
      return /*#__PURE__*/__reactCreateElement__(DashContext.Provider, {
        value: React.useMemo(() => ({
          styles,
          tokens: tokens[theme],
          theme,

          setTheme(nextTheme) {
            var _storedOnChange$curre2;

            if (nextTheme !== theme) (_storedOnChange$curre2 = storedOnChange.current) === null || _storedOnChange$curre2 === void 0 ? void 0 : _storedOnChange$curre2.call(storedOnChange, nextTheme);
            setTheme(nextTheme);
          },

          insertTokens(nextTokens) {
            for (var _themeName in tokens) {
              tokens[_themeName] = mergeTokens(cloneDeep(tokens[_themeName]), nextTokens);

              if (_themeName !== "default") {
                // @ts-expect-error: not sure
                tokens[_themeName] = mergeTokens(tokens[_themeName], themes[_themeName]);
              }
            }

            setTokenPointer(_ref9);
          },

          insertThemes(nextThemes) {
            for (var _themeName2 in nextThemes) {
              tokens[_themeName2] = mergeTokens(cloneDeep(tokens[_themeName2]), nextThemes[_themeName2]);
              themes[_themeName2] = mergeTokens(cloneDeep(themes[_themeName2]), nextThemes[_themeName2]);
            }

            setTokenPointer(_ref10);
          }

        }), [theme, tokenPointer]),
        children: children
      });
    }

  };
}
/**
 * A utility function that will compile style objects and callbacks into CSS strings.
 *
 * @param styles - A style callback, object, or string
 * @param tokens - A map of CSS tokens for style callbacks
 */

function compileStyles(styles, tokens) {
  var value = typeof styles === "function" ? styles(tokens) : styles;

  if (typeof value !== "string") {
    return value || {};
  }

  var cachedStyle = compileCache.get(value);
  if (cachedStyle) return cachedStyle;
  var stylePairs = [];
  var splitValue = value.split(";");

  for (var i = 0; i < splitValue.length; i++) {
    // Get prop name and prop value
    var match = propertyValuePattern.exec(splitValue[i]);
    if (match !== null) stylePairs.push([match[1], match[2]]);
  }

  try {
    var style = cssToRN(stylePairs);
    compileCache.set(value, style);
    return style;
  } catch (error) {
    var msg = error.message;

    if (msg.includes("Failed to parse declaration")) {
      var values = msg.replace("Failed to parse declaration ", "").replace(/"/g, "").trim().split(":");
      console.error("'" + values[0] + "' shorthand property requires units for example - " + values[0] + ": 20px or " + values[0] + ": 10px 20px 40px 50px");
    }

    var _style = {};
    compileCache.set(value, _style);
    return _style;
  }
}
var compileCache = /*#__PURE__*/new Map();

function mergeTokens(target, source) {
  for (var key in source) {
    var _target$key;

    var _value = source[key];
    target[key] = typeof _value === "object" && _value !== null && !Array.isArray(_value) ? mergeTokens((_target$key = target[key]) !== null && _target$key !== void 0 ? _target$key : {}, _value) : _value;
  }

  return target;
}

var propertyValuePattern = /\s*([^\s]+)\s*:\s*(.+?)\s*$/;

function compileLiterals(literals) {
  for (var _len4 = arguments.length, replacements = new Array(_len4 > 1 ? _len4 - 1 : 0), _key5 = 1; _key5 < _len4; _key5++) {
    replacements[_key5 - 1] = arguments[_key5];
  }

  return Array.isArray(literals) ? literals.reduce((curr, next, i) => curr + next + (replacements[i] || ""), "") : literals;
}

export { compileStyles, createStyles };
//# sourceMappingURL=index.dev.mjs.map
