"use strict";

exports.__esModule = true;
exports.compileStyles = compileStyles;
exports.createStyles = createStyles;

var _memoizeOne = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@essentials/memoize-one"));

var _cloneDeep = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("clone-deep"));

var _cssToReactNative = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("css-to-react-native"));

var React = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("react"));

var RN = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("react-native"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const __reactCreateElement__ = React.createElement;
const emptyObj = {};

function tokensAreEqual([t], [nt]) {
  return t === nt;
}

function _ref2(current) {
  return ++current;
}

function _ref3(current) {
  return ++current;
}

function createStyles(options = emptyObj) {
  var _options$themes, _options$tokens;

  const themes = (_options$themes = options.themes) !== null && _options$themes !== void 0 ? _options$themes : emptyObj;
  let currentTheme = "default";
  const tokens = {
    default: (_options$tokens = options.tokens) !== null && _options$tokens !== void 0 ? _options$tokens : emptyObj
  };

  if (options.themes) {
    const themeNames = Object.keys(options.themes);
    currentTheme = themeNames[0];

    for (const themeName of themeNames) {
      tokens[themeName] = mergeTokens((0, _cloneDeep.default)(tokens.default), themes[themeName]);
    }

    const colorScheme = RN.Appearance.getColorScheme();

    if (themeNames.length === 2 && "light" in tokens && "dark" in tokens && colorScheme && colorScheme in themes) {
      currentTheme = colorScheme;
    }
  }

  const isAutoThemeable = Object.values(tokens).length === 3 && "light" in tokens && "dark" in tokens;

  function cls(literals, ...placeholders) {
    literals = typeof literals === "function" ? (0, _memoizeOne.default)(literals) : literals;
    const styles = compileStyles(compileLiterals(literals, ...placeholders), tokens[currentTheme]);

    if (process.env.NODE_ENV !== "production") {
      return Object.freeze(styles);
    }

    return styles;
  }

  const styles = {
    variants(styleMap) {
      const memoMap = {};

      for (const key in styleMap) {
        // @ts-expect-error: key exists
        memoMap[key] = typeof styleMap[key] === "function" ? // @ts-expect-error: is assignable
        (0, _memoizeOne.default)(styleMap[key], tokensAreEqual) : styleMap[key];
      } // style('text', {})


      function style(...args) {
        const numArgs = args.length;
        let sheet = memoMap.default ? compileStyles(memoMap.default, tokens[currentTheme]) : {};

        if (numArgs === 1 && typeof args[0] === "string") {
          sheet = { ...sheet,
            ...compileStyles(memoMap[args[0]], tokens[currentTheme])
          };
        } else if (numArgs > 0) {
          let i = 0;
          let arg;
          sheet = { ...sheet
          };

          for (; i < numArgs; i++) {
            arg = args[i];

            if (typeof arg === "string") {
              Object.assign(sheet, compileStyles(memoMap[arg], tokens[currentTheme]));
            } else if (typeof arg === "object") {
              for (const key in arg) if (arg[key]) Object.assign(sheet, compileStyles(memoMap[key], tokens[currentTheme]));
            }
          }
        }

        return process.env.NODE_ENV !== "production" ? Object.freeze(sheet) : sheet;
      }

      style.styles = memoMap;
      return style;
    },

    one(literals, ...placeholders) {
      literals = typeof literals === "function" ? (0, _memoizeOne.default)(literals, tokensAreEqual) : literals;
      const compiledLiterals = compileLiterals(literals, ...placeholders);
      return function oneStyle(createStyle) {
        if (!createStyle && createStyle !== void 0) return emptyObj;
        let styles = compileStyles(compiledLiterals, tokens[currentTheme]);

        if (process.env.NODE_ENV !== "production") {
          styles = Object.freeze(styles);
        }

        return styles;
      };
    },

    cls,

    lazy(lazyFn) {
      return function (value) {
        if (value === undefined) return emptyObj;
        let styles = compileStyles(lazyFn(value), tokens[currentTheme]);

        if (process.env.NODE_ENV !== "production") {
          styles = Object.freeze(styles);
        }

        return styles;
      };
    },

    join(...css) {
      return cls("".concat(...css));
    },

    tokens,
    themes
  };
  const DashContext = /*#__PURE__*/React.createContext({
    styles,
    tokens: tokens[currentTheme],
    theme: currentTheme,

    setTheme(theme) {
      /* istanbul ignore next */
      console.error("DashContext was consumed outside of a Provider");
    },

    insertTokens(nextTokens) {
      /* istanbul ignore next */
      console.error("DashContext was consumed outside of a Provider");
    },

    insertThemes(nextThemes) {
      /* istanbul ignore next */
      console.error("DashContext was consumed outside of a Provider");
    }

  });
  /**
   * A hook for consuming dash context from the provider
   */

  function useDash() {
    return React.useContext(DashContext);
  }

  function styled(Component, baseStyles, areEqual) {
    const styles = typeof baseStyles === "function" && areEqual ? (0, _memoizeOne.default)(baseStyles, Array.isArray(areEqual) ? areEqual.length === 1 ? ([t, p], [nt, np]) => t === nt && p[areEqual[0]] === np[areEqual[1]] : ([t, p], [nt, np]) => t === nt && areEqual.every(key => p[key] === np[key]) : areEqual) : baseStyles;

    function compileRecursiveStyles(style, tokens) {
      function _ref(s) {
        return compileRecursiveStyles(s, tokens);
      }

      if (typeof style === "function" || typeof style === "string") {
        return compileStyles(style, tokens);
      } else if (Array.isArray(style)) {
        return Object.assign({}, ...style.map(_ref));
      }

      return style || {};
    }

    const RefForwardingComponent = /*#__PURE__*/React.forwardRef((props, ref) => {
      const dash = useDash();
      const baseStyle = !styles || typeof styles === "object" ? styles : compileStyles(typeof styles === "function" ? tokens => styles(tokens, props) : styles, dash.tokens);
      const outerStyle = typeof props.style === "function" || typeof props.style === "string" ? compileStyles(props.style, dash.tokens) : Array.isArray(props.style) ? compileRecursiveStyles(props.style, dash.tokens) : props.style;
      return /*#__PURE__*/__reactCreateElement__(Component, { ...props,
        style: outerStyle && baseStyle ? { ...baseStyle,
          ...outerStyle
        } : outerStyle ? outerStyle : baseStyle,
        ref: ref
      });
    });
    return RefForwardingComponent;
  }

  return {
    styles,
    styled,
    useDash,

    DashProvider({
      theme: controlledTheme,
      defaultTheme,
      onThemeChange,
      disableAutoThemeChange,
      children
    }) {
      const colorScheme = RN.useColorScheme();
      const didMount = React.useRef(false);
      const [userTheme, setTheme] = React.useState(() => {
        if (controlledTheme) {
          currentTheme = controlledTheme;
          return controlledTheme;
        }

        if (defaultTheme) {
          currentTheme = defaultTheme;
          return defaultTheme;
        }

        if (colorScheme && colorScheme in tokens && isAutoThemeable) {
          currentTheme = colorScheme;
          return colorScheme;
        }

        return currentTheme;
      });
      const [tokenPointer, setTokenPointer] = React.useState(0);
      const theme = controlledTheme !== null && controlledTheme !== void 0 ? controlledTheme : userTheme;
      currentTheme = theme;
      const storedOnChange = React.useRef(onThemeChange);
      React.useLayoutEffect(() => {
        storedOnChange.current = onThemeChange;
      });
      React.useLayoutEffect(() => {
        if (controlledTheme && controlledTheme !== userTheme) {
          setTheme(controlledTheme);
        }
      }, [controlledTheme, userTheme]);
      React.useLayoutEffect(() => {
        if (didMount.current && !controlledTheme && colorScheme && colorScheme !== theme && isAutoThemeable && !disableAutoThemeChange) {
          var _storedOnChange$curre;

          currentTheme = colorScheme;
          setTheme(currentTheme);
          (_storedOnChange$curre = storedOnChange.current) === null || _storedOnChange$curre === void 0 ? void 0 : _storedOnChange$curre.call(storedOnChange, currentTheme);
        }

        didMount.current = true;
      }, [colorScheme, controlledTheme]);
      return /*#__PURE__*/__reactCreateElement__(DashContext.Provider, {
        value: React.useMemo(() => ({
          styles,
          tokens: tokens[theme],
          theme,

          setTheme(nextTheme) {
            var _storedOnChange$curre2;

            if (nextTheme !== theme) (_storedOnChange$curre2 = storedOnChange.current) === null || _storedOnChange$curre2 === void 0 ? void 0 : _storedOnChange$curre2.call(storedOnChange, nextTheme);
            setTheme(nextTheme);
          },

          insertTokens(nextTokens) {
            for (const themeName in tokens) {
              tokens[themeName] = mergeTokens((0, _cloneDeep.default)(tokens[themeName]), nextTokens);

              if (themeName !== "default") {
                // @ts-expect-error: not sure
                tokens[themeName] = mergeTokens(tokens[themeName], themes[themeName]);
              }
            }

            setTokenPointer(_ref2);
          },

          insertThemes(nextThemes) {
            for (const themeName in nextThemes) {
              tokens[themeName] = mergeTokens((0, _cloneDeep.default)(tokens[themeName]), nextThemes[themeName]);
              themes[themeName] = mergeTokens((0, _cloneDeep.default)(themes[themeName]), nextThemes[themeName]);
            }

            setTokenPointer(_ref3);
          }

        }), [theme, tokenPointer]),
        children: children
      });
    }

  };
}
/**
 * A utility function that will compile style objects and callbacks into CSS strings.
 *
 * @param styles - A style callback, object, or string
 * @param tokens - A map of CSS tokens for style callbacks
 */


function compileStyles(styles, tokens) {
  const value = typeof styles === "function" ? styles(tokens) : styles;

  if (typeof value !== "string") {
    return value || {};
  }

  const cachedStyle = compileCache.get(value);
  if (cachedStyle) return cachedStyle;
  const stylePairs = [];
  const splitValue = value.split(";");

  for (let i = 0; i < splitValue.length; i++) {
    // Get prop name and prop value
    const match = propertyValuePattern.exec(splitValue[i]);
    if (match !== null) stylePairs.push([match[1], match[2]]);
  }

  try {
    const style = (0, _cssToReactNative.default)(stylePairs);
    compileCache.set(value, style);
    return style;
  } catch (error) {
    const msg = error.message;

    if (msg.includes("Failed to parse declaration")) {
      const values = msg.replace("Failed to parse declaration ", "").replace(/"/g, "").trim().split(":");
      console.error(`'${values[0]}' shorthand property requires units for example - ${values[0]}: 20px or ${values[0]}: 10px 20px 40px 50px`);
    }

    const style = {};
    compileCache.set(value, style);
    return style;
  }
}

const compileCache = /*#__PURE__*/new Map();

function mergeTokens(target, source) {
  for (const key in source) {
    var _target$key;

    const value = source[key];
    target[key] = typeof value === "object" && value !== null && !Array.isArray(value) ? mergeTokens((_target$key = target[key]) !== null && _target$key !== void 0 ? _target$key : {}, value) : value;
  }

  return target;
}

const propertyValuePattern = /\s*([^\s]+)\s*:\s*(.+?)\s*$/;

function compileLiterals(literals, ...replacements) {
  return Array.isArray(literals) ? literals.reduce((curr, next, i) => curr + next + (replacements[i] || ""), "") : literals;
}